#include "StdAfx.h"
#include "ReNamePlugInUtil.h"

#include "Util.h"
#include "ErrDef.h"

CReNamePlugInUtil::CReNamePlugInUtil(void)
{
	module = NULL;

	this->lockEvent = _CreateEvent(FALSE, TRUE, _T("Global\\EpgTimerSrv_ReName_Event") );
}

CReNamePlugInUtil::~CReNamePlugInUtil(void)
{
	UnInitialize();

	if( this->lockEvent != NULL ){
		UnLock();
		CloseHandle(this->lockEvent);
		this->lockEvent = NULL;
	}
}

BOOL CReNamePlugInUtil::Lock(LPCWSTR log, DWORD timeOut)
{
	if( this->lockEvent == NULL ){
		return FALSE;
	}
	if( log != NULL ){
		OutputDebugString(log);
	}
	DWORD dwRet = WaitForSingleObject(this->lockEvent, timeOut);
	if( dwRet == WAIT_ABANDONED || 
		dwRet == WAIT_FAILED){
		return FALSE;
	}
	return TRUE;
}

void CReNamePlugInUtil::UnLock(LPCWSTR log)
{
	if( this->lockEvent != NULL ){
		SetEvent(this->lockEvent);
	}
	if( log != NULL ){
		OutputDebugString(log);
	}
}

//DLL̏
//߂lF
// G[R[h
//F
// loadDllFilePath		[IN][hDLLpX
BOOL CReNamePlugInUtil::Initialize(
	LPCWSTR loadDllFilePath
	)
{
	if( module != NULL ){
		return FALSE;
	}
	if( Lock() == FALSE ) return FALSE;

	pfnGetPlugInNameRNP = NULL;
	pfnSettingRNP = NULL;
	pfnConvertRecNameRNP = NULL;
	pfnConvertRecName2RNP = NULL;


	BOOL ret = TRUE;

	module = ::LoadLibrary(loadDllFilePath);

	if( module == NULL ){
		OutputDebugString(L"dl̃[hɎs܂\r\n");
		UnLock();
		return FALSE;
	}

	pfnGetPlugInNameRNP = ( GetPlugInNameRNP ) ::GetProcAddress( module , "GetPlugInName");
	if( !pfnGetPlugInNameRNP ){
		OutputDebugString(L"GetPlugInName GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnSettingRNP = ( SettingRNP ) ::GetProcAddress( module , "Setting");
	if( !pfnSettingRNP ){
		OutputDebugString(L"Setting GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnConvertRecNameRNP = ( ConvertRecNameRNP ) ::GetProcAddress( module , "ConvertRecName");
	if( !pfnConvertRecNameRNP ){
		OutputDebugString(L"ConvertRecName GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnConvertRecName2RNP = ( ConvertRecName2RNP ) ::GetProcAddress( module , "ConvertRecName2");
	if( !pfnConvertRecName2RNP ){
		OutputDebugString(L"ConvertRecName GetProcAddress Ɏs\r\n");
	}

ERR_END:
	if( ret == FALSE ){
		::FreeLibrary( module );
		module=NULL;
	}
	UnLock();
	return ret;
}

//DLL̊J
//߂lF
// G[R[h
void CReNamePlugInUtil::UnInitialize()
{
	if( module != NULL ){
		::FreeLibrary( module );
	}
	module = NULL;

	pfnGetPlugInNameRNP = NULL;
	pfnSettingRNP = NULL;
	pfnConvertRecNameRNP = NULL;
	pfnConvertRecName2RNP = NULL;
}

//PlugIn̖O擾
//nameNULL͕KvȃTCYnameSizeŕԂ
//ʏnameSize=256ŌĂяo
//߂l
// TRUEijAFALSEisj
//F
// name						[OUT]
// nameSize					[IN/OUT]namẽTCY(WCHARP)
BOOL CReNamePlugInUtil::GetPlugInName(
	WCHAR* name,
	DWORD* nameSize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnGetPlugInNameRNP(name, nameSize);
}

//PlugInŐݒ肪KvȏꍇAݒp̃_CAOȂǂ\
//F
// parentWnd				[IN]eEChE
void CReNamePlugInUtil::Setting(
	HWND parentWnd
	)
{
	if( module == NULL ){
		return ;
	}

	return pfnSettingRNP(parentWnd);
}

//͂ꂽ\ɁA^掞̃t@C쐬igq܂ށj
//recNameNULL͕KvȃTCYrecNamesizeŕԂ
//ʏrecNamesize=256ŌĂяo
//߂l
// TRUEijAFALSEisj
//F
// info						[IN]\
// recName					[OUT]
// recNamesize				[IN/OUT]namẽTCY(WCHARP)
BOOL CReNamePlugInUtil::ConvertRecName(
	PLUGIN_RESERVE_INFO* info,
	WCHAR* recName,
	DWORD* recNamesize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}
	if( Lock() == FALSE ) return FALSE;

	BOOL ret = pfnConvertRecNameRNP(info, recName, recNamesize);

	UnLock();

	return ret;
}

//͂ꂽ\ɁA^掞̃t@C쐬igq܂ށj
//recNameNULL͕KvȃTCYrecNamesizeŕԂ
//ʏrecNamesize=256ŌĂяo
//߂l
// TRUEijAFALSEisj
//F
// info						[IN]\
// epgInfo					[IN]ԑgiEPG\Ŕԑg񂪑݂鎞A݂ȂꍇNULLj
// recName					[OUT]
// recNamesize				[IN/OUT]namẽTCY(WCHARP)
BOOL CReNamePlugInUtil::ConvertRecName2(
	PLUGIN_RESERVE_INFO* info,
	EPG_EVENT_INFO* epgInfo,
	WCHAR* recName,
	DWORD* recNamesize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}
	if( Lock() == FALSE ) return FALSE;
	BOOL ret = FALSE;
	if( pfnConvertRecName2RNP != NULL ){
		ret = pfnConvertRecName2RNP(info, epgInfo, recName, recNamesize);
	}else{
		ret = pfnConvertRecNameRNP(info, recName, recNamesize);
	}

	UnLock();

	return ret;
}

