#include "StdAfx.h"
#include "WritePlugInUtil.h"

#include "Util.h"
#include "ErrDef.h"

CWritePlugInUtil::CWritePlugInUtil(void)
{
	module = NULL;

	this->id = 0;
	this->lockEvent = _CreateEvent(FALSE, TRUE, NULL );
}

CWritePlugInUtil::~CWritePlugInUtil(void)
{
	UnInitialize();

	if( this->lockEvent != NULL ){
		UnLock();
		CloseHandle(this->lockEvent);
		this->lockEvent = NULL;
	}
}

BOOL CWritePlugInUtil::Lock(LPCWSTR log, DWORD timeOut)
{
	if( this->lockEvent == NULL ){
		return FALSE;
	}
	if( log != NULL ){
		OutputDebugString(log);
	}
	DWORD dwRet = WaitForSingleObject(this->lockEvent, timeOut);
	if( dwRet == WAIT_ABANDONED || 
		dwRet == WAIT_FAILED){
		return FALSE;
	}
	return TRUE;
}

void CWritePlugInUtil::UnLock(LPCWSTR log)
{
	if( this->lockEvent != NULL ){
		SetEvent(this->lockEvent);
	}
	if( log != NULL ){
		OutputDebugString(log);
	}
}

//DLL̏
//߂lF
// G[R[h
//F
// loadDllFilePath		[IN][hDLLpX
BOOL CWritePlugInUtil::Initialize(
	LPCWSTR loadDllFilePath
	)
{
	if( module != NULL ){
		return FALSE;
	}
	if( Lock() == FALSE ) return FALSE;

	pfnGetPlugInNameWP = NULL;
	pfnSettingWP = NULL;
	pfnCreateCtrlWP = NULL;
	pfnDeleteCtrlWP = NULL;
	pfnStartSaveWP = NULL;
	pfnStopSaveWP = NULL;
	pfnGetSaveFilePathWP = NULL;
	pfnAddTSBuffWP = NULL;


	BOOL ret = TRUE;

	module = ::LoadLibrary(loadDllFilePath);

	if( module == NULL ){
		OutputDebugString(L"dl̃[hɎs܂\r\n");
		UnLock();
		return FALSE;
	}

	pfnGetPlugInNameWP = ( GetPlugInNameWP ) ::GetProcAddress( module , "GetPlugInName");
	if( !pfnGetPlugInNameWP ){
		OutputDebugString(L"GetPlugInName GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnSettingWP = ( SettingWP ) ::GetProcAddress( module , "Setting");
	if( !pfnSettingWP ){
		OutputDebugString(L"Setting GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnCreateCtrlWP = ( CreateCtrlWP ) ::GetProcAddress( module , "CreateCtrl");
	if( !pfnCreateCtrlWP ){
		OutputDebugString(L"CreateCtrl GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnDeleteCtrlWP = ( DeleteCtrlWP ) ::GetProcAddress( module , "DeleteCtrl");
	if( !pfnDeleteCtrlWP ){
		OutputDebugString(L"DeleteCtrl GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnStartSaveWP = ( StartSaveWP ) ::GetProcAddress( module , "StartSave");
	if( !pfnStartSaveWP ){
		OutputDebugString(L"StartSave GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnStopSaveWP = ( StopSaveWP ) ::GetProcAddress( module , "StopSave");
	if( !pfnStopSaveWP ){
		OutputDebugString(L"StopSave GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnGetSaveFilePathWP = ( GetSaveFilePathWP ) ::GetProcAddress( module , "GetSaveFilePath");
	if( !pfnGetSaveFilePathWP ){
		OutputDebugString(L"GetSaveFilePath GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnAddTSBuffWP = ( AddTSBuffWP ) ::GetProcAddress( module , "AddTSBuff");
	if( !pfnAddTSBuffWP ){
		OutputDebugString(L"AddTSBuff GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}

	if( pfnCreateCtrlWP(&this->id) == FALSE ){
		this->id = 0;
		ret = FALSE;
		goto ERR_END;
	}

ERR_END:
	if( ret == FALSE ){
		::FreeLibrary( module );
		module=NULL;
	}
	UnLock();
	return ret;
}

//DLL̊J
//߂lF
// G[R[h
void CWritePlugInUtil::UnInitialize()
{
	if( module != NULL ){
		pfnDeleteCtrlWP(this->id);
		this->id = 0;
		::FreeLibrary( module );
	}
	module = NULL;
	
	pfnGetPlugInNameWP = NULL;
	pfnSettingWP = NULL;
	pfnCreateCtrlWP = NULL;
	pfnDeleteCtrlWP = NULL;
	pfnStartSaveWP = NULL;
	pfnStopSaveWP = NULL;
	pfnGetSaveFilePathWP = NULL;
	pfnAddTSBuffWP = NULL;

}

//PlugIn̖O擾
//nameNULL͕KvȃTCYnameSizeŕԂ
//ʏnameSize=256ŌĂяo
//߂l
// TRUEijAFALSEisj
//F
// name						[OUT]
// nameSize					[IN/OUT]namẽTCY(WCHARP)
BOOL CWritePlugInUtil::GetPlugInName(
	WCHAR* name,
	DWORD* nameSize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnGetPlugInNameWP(name, nameSize);
}

//PlugInŐݒ肪KvȏꍇAݒp̃_CAOȂǂ\
//F
// parentWnd				[IN]eEChE
void CWritePlugInUtil::Setting(
	HWND parentWnd
	)
{
	if( module == NULL ){
		return ;
	}

	return pfnSettingWP(parentWnd);
}

//t@CۑJn
//߂lF
// TRUEijAFALSEisj
//F
// fileName				[IN]ۑt@CtpXiKvɉĊgqςȂǍsj
// overWriteFlag		[IN]t@Cݎɏ㏑邩ǂiTRUEFAFALSEFȂj
// createSize			[IN]͗\zeʁi188oCgTSł̗eʁB^掞ȂǑԖ̏ꍇ0BȂǂ̉\̂Ŗڈxj
BOOL CWritePlugInUtil::StartSave(
	LPCWSTR fileName,
	BOOL overWriteFlag,
	ULONGLONG createSize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}
	if( Lock() == FALSE ) return FALSE;

	BOOL ret = pfnStartSaveWP(this->id, fileName, overWriteFlag, createSize);

	UnLock();

	return ret;
}

//t@CۑI
//߂lF
// TRUEijAFALSEisj
BOOL CWritePlugInUtil::StopSave(
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}
	if( Lock() == FALSE ) return FALSE;

	BOOL ret = pfnStopSaveWP(this->id);

	UnLock();

	return ret;
}

//ۂɕۑĂt@CpX擾iĐob`ɗpj
//filePathNULL͕KvȃTCYfilePathSizeŕԂ
//ʏfilePathSize=512ŌĂяo
//߂lF
// TRUEijAFALSEisj
//F
// filePath				[OUT]ۑt@CtpX
// filePathSize			[IN/OUT]filePath̃TCY(WCHARP)
BOOL CWritePlugInUtil::GetSaveFilePath(
	WCHAR* filePath,
	DWORD* filePathSize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}
	if( Lock() == FALSE ) return FALSE;

	BOOL ret = pfnGetSaveFilePathWP(this->id, filePath, filePathSize);

	UnLock();

	return ret;
}

//ۑpTSf[^𑗂
//󂫗eʕsȂǂŏosꍇAwriteSize̒l
//ēxۑƂ̑MJnn_߂
//߂lF
// TRUEijAFALSEisj
//F
// data					[IN]TSf[^
// size					[IN]datãTCY
// writeSize			[OUT]ۑɗpTCY
BOOL CWritePlugInUtil::AddTSBuff(
	BYTE* data,
	DWORD size,
	DWORD* writeSize
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}
	if( Lock() == FALSE ) return FALSE;

	BOOL ret = pfnAddTSBuffWP(this->id, data, size, writeSize);

	UnLock();

	return ret;
}

