#include "StdAfx.h"
#include "twitterUtil.h"

#include "Util.h"
#include "ErrDef.h"

CTwitterUtil::CTwitterUtil(void)
{
	module = NULL;

	this->id = 0;
	this->lockEvent = _CreateEvent(FALSE, TRUE, NULL );
}

CTwitterUtil::~CTwitterUtil(void)
{
	UnInitialize();

	if( this->lockEvent != NULL ){
		UnLock();
		CloseHandle(this->lockEvent);
		this->lockEvent = NULL;
	}
}

BOOL CTwitterUtil::Lock(LPCWSTR log, DWORD timeOut)
{
	if( this->lockEvent == NULL ){
		return FALSE;
	}
	if( log != NULL ){
		OutputDebugString(log);
	}
	DWORD dwRet = WaitForSingleObject(this->lockEvent, timeOut);
	if( dwRet == WAIT_ABANDONED || 
		dwRet == WAIT_FAILED){
		return FALSE;
	}
	return TRUE;
}

void CTwitterUtil::UnLock(LPCWSTR log)
{
	if( this->lockEvent != NULL ){
		SetEvent(this->lockEvent);
	}
	if( log != NULL ){
		OutputDebugString(log);
	}
}

//DLL̏
//߂lF
// G[R[h
DWORD CTwitterUtil::Initialize()
{
	if( module != NULL ){
		return FALSE;
	}
	if( Lock() == FALSE ) return FALSE;

	pfnInitializeTW = NULL;
	pfnUnInitializeTW = NULL;
	pfnGetIEProxyConfigTW = NULL;
	pfnGetProxyAutoDetectTW = NULL;
	pfnGetProxyAutoScriptTW = NULL;
	pfnSetProxyTW = NULL;
	pfnGetAuthorizationUrlTW = NULL;
	pfnSetAuthorizationPWDTW = NULL;
	pfnSendTweetTW = NULL;
	pfnGetTweetQueTW = NULL;


	BOOL ret = TRUE;

	module = ::LoadLibrary(L"twitter.dll");

	if( module == NULL ){
		UnLock();
		return FALSE;
	}

	pfnInitializeTW = ( InitializeTW ) ::GetProcAddress( module , "InitializeTW");
	if( !pfnInitializeTW ){
		OutputDebugString(L"InitializeTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnUnInitializeTW = ( UnInitializeTW ) ::GetProcAddress( module , "UnInitializeTW");
	if( !pfnUnInitializeTW ){
		OutputDebugString(L"UnInitializeTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnGetIEProxyConfigTW = ( GetIEProxyConfigTW ) ::GetProcAddress( module , "GetIEProxyConfigTW");
	if( !pfnGetIEProxyConfigTW ){
		OutputDebugString(L"GetIEProxyConfigTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnGetProxyAutoDetectTW = ( GetProxyAutoDetectTW ) ::GetProcAddress( module , "GetProxyAutoDetectTW");
	if( !pfnGetProxyAutoDetectTW ){
		OutputDebugString(L"GetProxyAutoDetectTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnGetProxyAutoScriptTW = ( GetProxyAutoScriptTW ) ::GetProcAddress( module , "GetProxyAutoScriptTW");
	if( !pfnGetProxyAutoScriptTW ){
		OutputDebugString(L"GetProxyAutoScriptTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnSetProxyTW = ( SetProxyTW ) ::GetProcAddress( module , "SetProxyTW");
	if( !pfnSetProxyTW ){
		OutputDebugString(L"SetProxyTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnGetAuthorizationUrlTW = ( GetAuthorizationUrlTW ) ::GetProcAddress( module , "GetAuthorizationUrlTW");
	if( !pfnGetAuthorizationUrlTW ){
		OutputDebugString(L"GetAuthorizationUrlTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}
	pfnSetAuthorizationPWDTW = ( SetAuthorizationPWDTW ) ::GetProcAddress( module , "SetAuthorizationPWDTW");
	if( !pfnSetAuthorizationPWDTW ){
		OutputDebugString(L"SetAuthorizationPWDTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}

	pfnSendTweetTW = ( SendTweetTW ) ::GetProcAddress( module , "SendTweetTW");
	if( !pfnSendTweetTW ){
		OutputDebugString(L"SendTweetTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}

	pfnGetTweetQueTW = ( GetTweetQueTW ) ::GetProcAddress( module , "GetTweetQueTW");
	if( !pfnGetTweetQueTW ){
		OutputDebugString(L"GetTweetQueTW GetProcAddress Ɏs\r\n");
		ret = FALSE;
		goto ERR_END;
	}

	if( pfnInitializeTW(&this->id) == FALSE ){
		this->id = 0;
		ret = FALSE;
		goto ERR_END;
	}

ERR_END:
	if( ret == FALSE ){
		::FreeLibrary( module );
		module=NULL;
	}
	UnLock();
	return ret;
}

//DLL̊J
//߂lF
// G[R[h
DWORD CTwitterUtil::UnInitialize()
{
	if( module != NULL ){
		pfnUnInitializeTW(this->id);
		this->id = 0;
		::FreeLibrary( module );
	}
	module = NULL;
	
	pfnInitializeTW = NULL;
	pfnUnInitializeTW = NULL;
	pfnGetIEProxyConfigTW = NULL;
	pfnGetProxyAutoDetectTW = NULL;
	pfnGetProxyAutoScriptTW = NULL;
	pfnSetProxyTW = NULL;
	pfnGetAuthorizationUrlTW = NULL;
	pfnSetAuthorizationPWDTW = NULL;
	pfnSendTweetTW = NULL;
	pfnGetTweetQueTW = NULL;

	return NO_ERR;
}

//IEProxyݒ擾
//߂lF
// TRUEi֐jAFALSEi֐sj
//F
// proxyConfig		[OUT]IEProxyݒ̏iAPIďo܂Ńmہj
BOOL CTwitterUtil::GetIEProxyConfig(
	CURRENT_USER_IE_PROXY_CONFIG** proxyConfig
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnGetIEProxyConfigTW(this->id, proxyConfig);
}

//IɌoProxỹAhX擾
//߂lF
// TRUEi֐jAFALSEi֐sj
//F
// proxyConfig		[OUT]Proxy̏iAPIďo܂Ńmہj
BOOL CTwitterUtil::GetProxyAutoDetect(
	PROXY_CONFIG** proxyConfig
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnGetProxyAutoDetectTW(this->id, proxyConfig);
}

//\XNvgProxỹAhX擾
//߂lF
// TRUEi֐jAFALSEi֐sj
//F
// scriptURL		[IN]\XNvgURL
// proxyConfig		[OUT]Proxy̏iAPIďo܂Ńmہj
BOOL CTwitterUtil::GetProxyAutoScript(
	LPCWSTR scriptURL,
	PROXY_CONFIG** proxyConfig
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnGetProxyAutoScriptTW(this->id, scriptURL, proxyConfig);
}

//Proxygpݒ
//߂lF
// G[R[h
//F
// useProxy			[IN]ProxygǂiTRUE:Proxygj
// proxyInfo		[IN]Proxygꍇ̐ݒ
void CTwitterUtil::SetProxy(
	BOOL useProxy,
	USE_PROXY_INFO* proxyInfo
	)
{
	if( module == NULL ){
		return ;
	}

	return pfnSetProxyTW(this->id, useProxy, proxyInfo);
}

//FؗpOCURL擾
//߂lF
// G[R[h
//F
// url			[OUT]FؗpOCURLiAPIďo܂Ńmہj
DWORD CTwitterUtil::GetAuthorizationUrl(
	WCHAR** url
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnGetAuthorizationUrlTW(this->id, url);
}

//F،ʂ̈Ïؔԍݒ肷
//GetAuthorizationUrlŎ擾lgp̂ŁAA̗Őݒ肷Kv
//߂lF
// G[R[h
//F
// password		[IN]Ïؔԍ
DWORD CTwitterUtil::SetAuthorizationPWD(
	LPCWSTR password
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnSetAuthorizationPWDTW(this->id, password);
}

//cC[g
//߂lF
// G[R[h
//F
// asyncMode	[IN]񓯊őM
// text			[IN]cC[ge
DWORD CTwitterUtil::SendTweet(
	BOOL asyncMode,
	LPCWSTR text
	)
{
	if( module == NULL ){
		return ERR_NOT_INIT;
	}

	return pfnSendTweetTW(this->id, asyncMode, text);
}

//񓯊cC[g̎c擾
//߂lF
// 
DWORD CTwitterUtil::GetTweetQue(
	)
{
	if( module == NULL ){
		return 0;
	}

	return pfnGetTweetQueTW(this->id);
}
